# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from .common import OperationMetadata
from .cross_network_automation import (
    ConnectionErrorType,
    CreateServiceConnectionMapRequest,
    CreateServiceConnectionPolicyRequest,
    CreateServiceConnectionTokenRequest,
    DeleteServiceClassRequest,
    DeleteServiceConnectionMapRequest,
    DeleteServiceConnectionPolicyRequest,
    DeleteServiceConnectionTokenRequest,
    GetServiceClassRequest,
    GetServiceConnectionMapRequest,
    GetServiceConnectionPolicyRequest,
    GetServiceConnectionTokenRequest,
    Infrastructure,
    IPVersion,
    ListServiceClassesRequest,
    ListServiceClassesResponse,
    ListServiceConnectionMapsRequest,
    ListServiceConnectionMapsResponse,
    ListServiceConnectionPoliciesRequest,
    ListServiceConnectionPoliciesResponse,
    ListServiceConnectionTokensRequest,
    ListServiceConnectionTokensResponse,
    ServiceClass,
    ServiceConnectionMap,
    ServiceConnectionPolicy,
    ServiceConnectionToken,
    UpdateServiceClassRequest,
    UpdateServiceConnectionMapRequest,
    UpdateServiceConnectionPolicyRequest,
)
from .hub import (
    AcceptHubSpokeRequest,
    AcceptHubSpokeResponse,
    AcceptSpokeUpdateRequest,
    AcceptSpokeUpdateResponse,
    AutoAccept,
    CreateHubRequest,
    CreateSpokeRequest,
    DeleteHubRequest,
    DeleteSpokeRequest,
    GetGroupRequest,
    GetHubRequest,
    GetRouteRequest,
    GetRouteTableRequest,
    GetSpokeRequest,
    Group,
    Hub,
    HubStatusEntry,
    LinkedInterconnectAttachments,
    LinkedProducerVpcNetwork,
    LinkedRouterApplianceInstances,
    LinkedVpcNetwork,
    LinkedVpnTunnels,
    ListGroupsRequest,
    ListGroupsResponse,
    ListHubSpokesRequest,
    ListHubSpokesResponse,
    ListHubsRequest,
    ListHubsResponse,
    ListRoutesRequest,
    ListRoutesResponse,
    ListRouteTablesRequest,
    ListRouteTablesResponse,
    ListSpokesRequest,
    ListSpokesResponse,
    LocationFeature,
    LocationMetadata,
    NextHopInterconnectAttachment,
    NextHopRouterApplianceInstance,
    NextHopVpcNetwork,
    NextHopVPNTunnel,
    PolicyMode,
    PresetTopology,
    PscPropagationStatus,
    QueryHubStatusRequest,
    QueryHubStatusResponse,
    RejectHubSpokeRequest,
    RejectHubSpokeResponse,
    RejectSpokeUpdateRequest,
    RejectSpokeUpdateResponse,
    Route,
    RouterApplianceInstance,
    RouteTable,
    RouteType,
    RoutingVPC,
    Spoke,
    SpokeSummary,
    SpokeType,
    State,
    UpdateGroupRequest,
    UpdateHubRequest,
    UpdateSpokeRequest,
)
from .policy_based_routing import (
    CreatePolicyBasedRouteRequest,
    DeletePolicyBasedRouteRequest,
    GetPolicyBasedRouteRequest,
    ListPolicyBasedRoutesRequest,
    ListPolicyBasedRoutesResponse,
    PolicyBasedRoute,
)

__all__ = (
    "OperationMetadata",
    "CreateServiceConnectionMapRequest",
    "CreateServiceConnectionPolicyRequest",
    "CreateServiceConnectionTokenRequest",
    "DeleteServiceClassRequest",
    "DeleteServiceConnectionMapRequest",
    "DeleteServiceConnectionPolicyRequest",
    "DeleteServiceConnectionTokenRequest",
    "GetServiceClassRequest",
    "GetServiceConnectionMapRequest",
    "GetServiceConnectionPolicyRequest",
    "GetServiceConnectionTokenRequest",
    "ListServiceClassesRequest",
    "ListServiceClassesResponse",
    "ListServiceConnectionMapsRequest",
    "ListServiceConnectionMapsResponse",
    "ListServiceConnectionPoliciesRequest",
    "ListServiceConnectionPoliciesResponse",
    "ListServiceConnectionTokensRequest",
    "ListServiceConnectionTokensResponse",
    "ServiceClass",
    "ServiceConnectionMap",
    "ServiceConnectionPolicy",
    "ServiceConnectionToken",
    "UpdateServiceClassRequest",
    "UpdateServiceConnectionMapRequest",
    "UpdateServiceConnectionPolicyRequest",
    "ConnectionErrorType",
    "Infrastructure",
    "IPVersion",
    "AcceptHubSpokeRequest",
    "AcceptHubSpokeResponse",
    "AcceptSpokeUpdateRequest",
    "AcceptSpokeUpdateResponse",
    "AutoAccept",
    "CreateHubRequest",
    "CreateSpokeRequest",
    "DeleteHubRequest",
    "DeleteSpokeRequest",
    "GetGroupRequest",
    "GetHubRequest",
    "GetRouteRequest",
    "GetRouteTableRequest",
    "GetSpokeRequest",
    "Group",
    "Hub",
    "HubStatusEntry",
    "LinkedInterconnectAttachments",
    "LinkedProducerVpcNetwork",
    "LinkedRouterApplianceInstances",
    "LinkedVpcNetwork",
    "LinkedVpnTunnels",
    "ListGroupsRequest",
    "ListGroupsResponse",
    "ListHubSpokesRequest",
    "ListHubSpokesResponse",
    "ListHubsRequest",
    "ListHubsResponse",
    "ListRoutesRequest",
    "ListRoutesResponse",
    "ListRouteTablesRequest",
    "ListRouteTablesResponse",
    "ListSpokesRequest",
    "ListSpokesResponse",
    "LocationMetadata",
    "NextHopInterconnectAttachment",
    "NextHopRouterApplianceInstance",
    "NextHopVpcNetwork",
    "NextHopVPNTunnel",
    "PscPropagationStatus",
    "QueryHubStatusRequest",
    "QueryHubStatusResponse",
    "RejectHubSpokeRequest",
    "RejectHubSpokeResponse",
    "RejectSpokeUpdateRequest",
    "RejectSpokeUpdateResponse",
    "Route",
    "RouterApplianceInstance",
    "RouteTable",
    "RoutingVPC",
    "Spoke",
    "SpokeSummary",
    "UpdateGroupRequest",
    "UpdateHubRequest",
    "UpdateSpokeRequest",
    "LocationFeature",
    "PolicyMode",
    "PresetTopology",
    "RouteType",
    "SpokeType",
    "State",
    "CreatePolicyBasedRouteRequest",
    "DeletePolicyBasedRouteRequest",
    "GetPolicyBasedRouteRequest",
    "ListPolicyBasedRoutesRequest",
    "ListPolicyBasedRoutesResponse",
    "PolicyBasedRoute",
)
